@@ -95,6 +95,9 @@ GstStructure *          gst_structure_new                  (const gchar *
 GstStructure *          gst_structure_new_valist           (const gchar *            name,
 						            const gchar *            firstfield,
 							    va_list                  varargs);
+GstStructure *          gst_structure_id_new               (GQuark                   name_quark,
+                                                            GQuark                   field_quark,
+                                                            ...);
 GstStructure *          gst_structure_copy                 (const GstStructure      *structure);
 void			gst_structure_set_parent_refcount  (GstStructure            *structure,
                                                             gint            *refcount);
@@ -129,6 +132,21 @@ void                    gst_structure_id_set_valist         (GstStructure
 							    GQuark                   fieldname,
 							    va_list varargs);
 
+gboolean                gst_structure_get_valist           (GstStructure            *structure,
+                                                            const char              *first_fieldname,
+                                                            va_list                  args);
+
+gboolean                gst_structure_get                  (GstStructure            *structure,
+                                                            const char              *first_fieldname,
+                                                            ...) G_GNUC_NULL_TERMINATED;
+
+gboolean                gst_structure_id_get_valist        (GstStructure            *structure,
+                                                            GQuark                   first_field_id,
+                                                            va_list                  args);
+
+gboolean                gst_structure_id_get               (GstStructure            *structure,
+                                                            GQuark                   first_field_id,
+                                                            ...) G_GNUC_NULL_TERMINATED;
 
 G_CONST_RETURN GValue * gst_structure_id_get_value         (const GstStructure      *structure,
 							    GQuark                   field);
@@ -154,6 +172,11 @@ gboolean                gst_structure_map_in_place         (GstStructure
 							    gpointer                 user_data);
 gint                    gst_structure_n_fields             (const GstStructure      *structure);
 const gchar *           gst_structure_nth_field_name       (const GstStructure      *structure, guint index);
+gboolean                gst_structure_id_has_field         (const GstStructure      *structure,
+							    GQuark                   field);
+gboolean                gst_structure_id_has_field_typed   (const GstStructure      *structure,
+							    GQuark                   field,
+							    GType                    type);
 gboolean                gst_structure_has_field            (const GstStructure      *structure,
 							    const gchar             *fieldname);
 gboolean                gst_structure_has_field_typed      (const GstStructure      *structure,
@@ -167,6 +190,9 @@ gboolean                gst_structure_get_boolean          (const GstStructure
 gboolean                gst_structure_get_int              (const GstStructure      *structure,
 							    const gchar             *fieldname,
 							    gint                    *value);
+gboolean                gst_structure_get_uint             (const GstStructure      *structure,
+							    const gchar             *fieldname,
+							    guint                   *value);
 gboolean                gst_structure_get_fourcc           (const GstStructure      *structure,
 							    const gchar             *fieldname,
 							    guint32                 *value);
